<?php

use Monolog\Handler\StreamHandler;
use Monolog\Logger;
use datafast\payment\model\Constants;
use datafast\payment\PaymentService;
use datafast\payment\datafast\payment\Config;
use datafast\payment\datafast\payment\model\Message;
use datafast\payment\datafast\payment\model\PaymentResponse;

include_once(_PS_MODULE_DIR_ . 'datafast/vendor/autoload.php');

class datafastResultModuleFrontController extends ModuleFrontController
{

    public function postProcess()
    {
        parent::initContent();

            if (!$this->isPaymentMethodValid()) {
                die($this->module->l('This payment method is not available.', 'datafast'));
            }

            $cart = $this->context->cart;
            if (!$cart || !$cart->id_customer || !$cart->id_address_delivery || !$cart->id_address_invoice) {
                $this->redirectTo('order', array('step' => 1));
            }

            $customer = new Customer($cart->id_customer);
            if (!Validate::isLoadedObject($customer)) {
                $this->redirectTo('order', array('step' => 1));
            }

            $secureKey = $customer->secure_key;

            $paymentService = new PaymentService();

            $config = new Config();
			 
            $data['DATAFAST_DEV']=Configuration::get('DATAFAST_DEV', null);
            $data['DATAFAST_BEARER_TOKEN']=Configuration::get('DATAFAST_BEARER_TOKEN', null);
            $data['DATAFAST_ENTITY_ID']=Configuration::get('DATAFAST_ENTITY_ID', null);
            $data['DATAFAST_MID']=Configuration::get('DATAFAST_MID', null);
            $data['DATAFAST_TID']=Configuration::get('DATAFAST_TID', null);
            $data['DATAFAST_RISK']=Configuration::get('DATAFAST_RISK', null);
            $data['DATAFAST_PROVEEDOR']=Configuration::get('DATAFAST_PROVEEDOR', null);
			$data['DATAFAST_PREFIJOTRX']=Configuration::get('DATAFAST_PREFIJOTRX', null);
			
            $request = $config->getDatafastRequest($data);
            
            
            $cart_id= $this->context->cart->id;
            $order_id = (int)Tools::getValue('order_id');
            $payment_id = trim(Tools::getValue('MD'));

            
            $checkOutId = Tools::getValue('id');
            $resourcePathUri = str_replace("{id}", $checkOutId, $request->getResourcePathUri());

            $this->getLogger()->debug("resourcePathUri--->" . $resourcePathUri);
            $request->setResourcePathUri($resourcePathUri);
            
			$paymentResp = $paymentService->processPayment($request);
         
            $objRequest =  json_decode($request, true);
            $objResponse =  json_decode($paymentResp, true);
            $resultCode = $objResponse["result"]["code"];
            $resultdescription=  str_replace("'","\'",$objResponse['result']['description']);
            $accepted = $this->validateTransaction($resultCode);

            $total = (float)$cart->getOrderTotal(true, Cart::BOTH);
            $module_name = $this->module->displayName;
                
            $this->addTransaction($cart_id,$customer->id,$total,$accepted,"",$objResponse);
			
			$datafastExtendedDescripcion = $objResponse['resultDetails']['ExtendedDescription'];
			if ($datafastExtendedDescripcion == "")
            {
                $datafastExtendedDescripcion = $resultdescription;   
            }


            if ($accepted) {
				 
                $payment_status = Configuration::get('PS_OS_PAYMENT', null);
                $message = $this->l('El pago se registró correctamente');
                
                $datafastBrand = $objResponse['paymentBrand'];
                $datafastAmount = $objResponse['amount'];
                $datafastAuth = $objResponse['resultDetails']['AuthCode'];
                $datafastCardHolder = $objResponse['card']['holder'];
				
				
				Context::getContext()->cookie->datafastBrand = $datafastBrand;
                Context::getContext()->cookie->datafastAmount = $datafastAmount;
                Context::getContext()->cookie->datafastAuth = $datafastAuth;
                Context::getContext()->cookie->datafastCardHolder = $datafastCardHolder;
				Context::getContext()->cookie->datafastExtendedDescripcion = $datafastExtendedDescripcion;
				
                $this->module->validateOrder((int)$cart_id, $payment_status, $total, $module_name, $message, array(), (int)$cart->id_currency, false, $secureKey);
                
                $this->redirectTo('order-confirmation', array(
                    'id_cart' => (int)$cart_id,
                    'id_module' => (int)$this->module->id,
                    'id_order' => (int)$this->module->currentOrder,
                    'key' => $customer->secure_key
                ));
            } else {
				
                $logInfo = "Error en el pago  de:  " . $total . ", customer id: " . $customer->id . " cart id: " . $cart->id . " order id" . $this->module->currentOrder . " Detalle: " . $datafastExtendedDescripcion;
                $this->getLogger()->info($logInfo);

                Context::getContext()->cookie->errorMessage = $datafastExtendedDescripcion;
                Tools::redirect(Context::getContext()->link->getModuleLink('datafast', 'error', array()));
            }
 
    }

    private function isPaymentMethodValid()
    {
        if (!$this->module->active) {
            return false;
        }

        if (method_exists('Module', 'getPaymentModules')) {
            foreach (Module::getPaymentModules() as $module) {
                if (isset($module['name']) && $module['name'] === $this->module->name) {
                    return true;
                }
            }
        } else {
            return true;
        }

        return false;
    }

    private function redirectTo($controller, array $params = array())
    {
        $query_string = !empty($params) ? http_build_query($params) : '';

        Tools::redirect('index.php?controller=' . $controller . '&' . $query_string);

    }

    /**
     * @return Logger
     */
    private function getLogger(): Logger
    {
        $logger = new Logger('PaymentFrontController');
        $logger->pushHandler(new StreamHandler(Constants::LOGGER_FILE, Logger::DEBUG));
        return $logger;
    }

    private function validateTransaction(string $resultCode): bool
    {
		
        $testMode = Configuration::get('DATAFAST_DEV', null);
        $validTransaction = false;

        if (in_array($resultCode, Constants::TRANSACTION_APPROVED_TEST) && $testMode) {
            $validTransaction = true;
        }
        if ($resultCode == Constants::TRANSACTION_APPROVED_PROD && !$testMode) {
            $validTransaction = true;
        }
        return $validTransaction;
    }

    private function addTransaction($cart_id,$customer_id,$amount,$accepted,$paymentRequest,$paymentResponse)
    {        
        $resultCode = $paymentResponse["result"]["code"];
		$message = "";
 
        $transaction_id = '';
        $amount = 0;
        $transaction_id = '';
        $payment_type = '';
        $payment_brand = '';
        $merchant_transactionId= '';
        $result_code = '';
        $extended_description= '';
        $acquirer_response = '';
        $batch_no = '';
        $total_amount = '';
        $reference_no = '';
        $bin = '';
        $last_4_Digits = '';
        $email = '';
        $shopper_mid= '';
        $shopper_tid = '';
        $timestamp = '';
        $response = '';

        $checkout_id = $paymentResponse['id'];
        $result_code = $paymentResponse['result']['code'];
        $result_description=  str_replace("'","\'",$paymentResponse['result']['description']);
        $request_json = str_replace("'","\'",json_encode($paymentRequest));   
        $response_json = str_replace("'","\'",json_encode($paymentResponse));   
        $timestamp = $paymentResponse['timestamp'];
        
        $status = "0";
               

            $amount = $paymentResponse['amount'];

            $transaction_id = $paymentResponse['id'];
            $payment_type = $paymentResponse['paymentType'];
            $payment_brand = $paymentResponse['paymentBrand'];
            $amount = $paymentResponse['amount'];
            $merchant_transactionId= $paymentResponse['merchantTransactionId'];
            
            $extended_description=  str_replace("'","\'",$paymentResponse['resultDetails']['ExtendedDescription']);
            if ($extended_description == "")
            {
             $extended_description = $result_description;   
            }
            $response = $paymentResponse['resultDetails']['Response'];
            $acquirer_response = $paymentResponse['resultDetails']['AcquirerResponse'];
            $auth_code = $paymentResponse['resultDetails']['AuthCode'];
            $acquirer_code = $paymentResponse['resultDetails']['AcquirerCode'];
            $batch_no = $paymentResponse['resultDetails']['BatchNo'];
            $interest = $paymentResponse['resultDetails']['Interest'];
            $total_amount = $paymentResponse['resultDetails']['TotalAmount'];
            $reference_no = $paymentResponse['resultDetails']['ReferenceNo'];
            $bin = $paymentResponse['card']['bin'];
            $last_4_Digits = $paymentResponse['card']['last4Digits'];
            $email =  str_replace("'","\'",$paymentResponse['customer']['email']);
            $shopper_mid =  $paymentResponse['customParameters']['SHOPPER_MID'];
            $shopper_tid = $paymentResponse['customParameters']['SHOPPER_TID'];
        
        
        if ($accepted) {  

            $registrationId = $paymentResponse['registrationId'];
            $status = "1";
     
            if ($registrationId <> "")
            {

                $sqlCount = '
                SELECT COUNT(tkn.id)    AS CountToken
                FROM `' . _DB_PREFIX_ . 'datafast_customertoken` tkn
                WHERE tkn.`token` = \'' .$registrationId . '\'';
                $countToken = Db::getInstance()->executeS($sqlCount);  
                    $countTkn = $countToken[0]['CountToken'];

            
                if ($countTkn == 0)
                {
                    $query = 'INSERT INTO ' . _DB_PREFIX_ . 'datafast_customertoken
                    (`customer_id`, 
                    `token`, 
                    `status`,
                    `updated_at`)
                        VALUES ( 
                                \'' . $customer_id . '\',
                                \'' .$registrationId . '\',
                                \'1\',
                                \'' . date('Y-m-d H:i:s') . '\'
                                )';
                    Db::getInstance()->execute($query);
                }
            }
        }
 
        $query = 'INSERT INTO ' . _DB_PREFIX_ . 'datafast_transactions
                             (`cart_id`, 
                             `customer_id`, 
                             `transaction_id`, 
                             `checkout_id`, 
                             `payment_type`,
                             `payment_brand`,
                             `amount`,
                             `merchant_transactionId`,
                             `result_code`,
                             `result_description`,
                             `extended_description`,
                             `acquirer_response`,
                             `response`,
                             `auth_code`,
                             `acquirer_code`,
                             `batch_no`,
                             `interest`,
                             `total_amount`,
                             `reference_no`,
                             `bin`,
                             `last_4_Digits`,
                             `email`,
                             `shopper_mid`,
                             `shopper_tid`,
                             `timestamp`,
                             `request_json`,
                             `response_json`,
                             `status`,
                             `updated_at`)
                                 VALUES ( 
                                    ' . $cart_id . ',
                                    \'' . $customer_id . '\',
                                    \'' .$transaction_id . '\',
                                    \'' .$checkout_id . '\',
                                    \'' . $payment_type . '\',
                                    \'' . $payment_brand . '\',
                                    \'' . $amount . '\',
                                    \'' . $merchant_transactionId . '\',
                                    \'' . $result_code . '\',
                                    \'' . $result_description . '\',
                                    \'' . $extended_description . '\',
                                    \'' . $acquirer_response . '\',
                                    \'' . $response . '\',
                                    \'' . $auth_code . '\',
                                    \'' . $acquirer_code . '\',
                                    \'' . $batch_no . '\',
                                    \'' . $interest . '\',
                                    \'' . $total_amount . '\',
                                    \'' . $reference_no . '\',
                                    \'' . $bin . '\',
                                    \'' . $last_4_Digits . '\',
                                    \'' . $email . '\',
                                    \'' . $shopper_mid . '\',
                                    \'' . $shopper_tid . '\',
                                    \'' . $timestamp . '\',
                                    \'' . $request_json . '\',
                                    \'' . $response_json . '\',
                                    \'' . $status . '\',
                                    \'' . date('Y-m-d H:i:s') . '\'
                         )';
        Db::getInstance()->execute($query);
       
    }

    private function getDetailErrorMessage(string $resultCode): string
    {
        return Message::getClientMessageDescription($resultCode);
    }
}

